#include "includes.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdio.h>

#include "asndef.h"
#include "snmp_impl_def.h"
#include "nmbd_snmp.h"

int ClientSNMPcon=-1;
int ClientSNMP=-1;

extern int DEBUGLEVEL;
extern time_t StartupTime;
extern int  updatecount;
extern fstring global_myworkgroup;
extern uint16 samba_nb_type; /* Samba's NetBIOS name type. */
extern pstring global_myname; 
extern BOOL found_lm_clients;
extern int global_nmb_port;
extern int DEBUGLEVEL;
extern pstring debugf;
extern pstring myhostname;

/*
fstring myworkgroup;
BOOL append_log;
BOOL global_in_nmbd;
FILE *dbf;
char **my_netbios_names;
char *optarg;
fstring local_machine;
fstring remote_machine;
int ClientDGRAM;
int ClientNMB;
int ClientSNMP;
int ClientSNMPcon;
int num_response_packets;
pstring scope;
struct in_addr allones_ip;
struct in_addr ipzero;
struct in_addr loopback_ip;
struct ubi_dlList lmb_browserlist[];
*/

int
open_un_socket_in(char *name)
{
   struct sockaddr_un server;
   int sock;

   sock = socket(AF_UNIX, SOCK_STREAM, 0);
   if (sock < 0) {
      DEBUG(0,("opening stream socket"));
      return -1;
   }
   /* Name socket using file system name. */
   server.sun_family = AF_UNIX;
   safe_strcpy(server.sun_path, name,sizeof(server.sun_path));
   if (bind(sock, (struct sockaddr *)&server, SUN_LEN(&server))) {
      DEBUG(0,("error binding stream socket"));
      return -1;
   }
   return sock;
}

int
SNMPCheckError(int err)
{
	if(err==EPIPE) {
		close(ClientSNMP);
		DEBUG(0,("Connection closed\n"));
		ClientSNMP=-1;
		return -1;
	}
	return -1;
}

int
SNMPCloseAll(void)
{
	int err=0;

	if(ClientSNMPcon>0) {
		err=close(ClientSNMPcon);
		ClientSNMPcon=-1;
		unlink(UN_SOCKET_NAME);
	}

	if(ClientSNMP>0) {
		err|=close(ClientSNMP);
		ClientSNMP=-1;
	}
	return err;
}
/*
int
SNMPGetPort(void ) 
{
	struct servent *serv;

	serv=getservbyname("samba-snmp","tcp");
	if(serv==0) {
		DEBUG(0,("Error getting service '%s'\n","samba-snmp"));
		return 0;
	}
	return ntohs(serv->s_port);
}
*/

int 
SNMPInit(void)
{
	struct timeval tv;
	int retval;

	int port;

	ClientSNMPcon=-1;
	ClientSNMP=-1;
/*
	port=SNMPGetPort();
	if(port==0) {
		return -1;
	}
*/


/*	ClientSNMPcon=open_socket_in(SOCK_STREAM, port, 0,0, -1); */
	ClientSNMPcon=open_un_socket_in(UN_SOCKET_NAME);
	if(ClientSNMPcon<=0)
		return -1;

	if(listen(ClientSNMPcon,1)<0) {
		DEBUG(0,("Error listening on port %d\n",port));
		return -1;
	}
	return ClientSNMPcon;
}

int
SNMPAccept(void)
{
	struct sockaddr addr;
	int ilen;

	if(ClientSNMP>0) {
		DEBUG(0,("SNMP Client already connected\n"));
		errno=ECONNREFUSED;
		return -1;
	}

	ilen=sizeof(addr);
	ClientSNMP=accept(ClientSNMPcon,&addr,&ilen);

	if(ClientSNMP==-1) {
		DEBUG(0,("SNMPAccept error: %s\n",strerror(errno)));
		return -1;
	}
	DEBUG(0,("Accept\n"));
	return ClientSNMP;
}

int
SNMPBuildAnswer(struct snmp_answer * ans)
{

	ans->type=snmp_types[ans->item];

	switch(ans->item) {

	case NMB_SNMP_UPTIME :
	{
		time_t t;

		t=time(NULL);
		ans->data.t=t-StartupTime;
		break;
	}
	case NMB_SNMP_UPDATECOUNT :
		ans->data.l=(long) updatecount;
		break;
	case NMB_SNMP_WORKGROUP :
		safe_strcpy(ans->data.s,global_myworkgroup,sizeof(ans->data.s));
		break;
	case NMB_SNMP_NB_TYPE : /* Samba's NetBIOS name type. */
		ans->data.ul=(unsigned long ) samba_nb_type;
		break;
	case NMB_SNMP_NB_NAME :
		safe_strcpy(ans->data.s,global_myname,sizeof(ans->data.s));
		break;
	case NMB_SNMP_HOSTNAME :
		safe_strcpy(ans->data.s,myhostname,sizeof(ans->data.s));
		break;
	case NMB_SNMP_LM_CLIENTS :
		ans->data.l=(long) found_lm_clients;
		break;
	case NMB_SNMP_NMB_PORT :
		ans->data.l=(long) global_nmb_port;
		break;
	case NMB_SNMP_DEBUGLEVEL :
		ans->data.l=(long) DEBUGLEVEL;
		break;
	case NMB_SNMP_DEBUGFILE :
		safe_strcpy(ans->data.s,debugf,sizeof(ans->data.s));
		break;
	default:
		ans->type=-1;
		DEBUG(0,("Invalid item %d requested\n",ans->item));
	};

	return 0;
}

int
SNMPAnswerRequest(void)
{
	struct snmp_answer ans;
	int  request;
	int  num,len;

	len=recv(ClientSNMP,&request,sizeof(request),0);
	switch (len) {
		case -1:
			DEBUG(0,("Error '%s' (%d) while SNMP request\n",
				strerror(errno),errno));

			return SNMPCheckError(errno);
			break;
		case 0:
			/* I got a empty message, probably the socket
			   was closed on the client side */
			/* I don't know how to check this */
			DEBUG(0,("Got empty message\nConnection closed\n"));
			close(ClientSNMP);
			ClientSNMP=-1;
			return -1;
			break;
		case 1:
			DEBUG(0,("Wrong request size\n"));
			return -1;
			break;
	}
	DEBUG(0,("answer %d\n",len));

	ans.item=request;
	len=SNMPBuildAnswer(&ans);
	if (len<0) {
		DEBUG(0,("Wrong SNMP request %d\n",ans.item));
	}

	num=send(ClientSNMP,&ans,sizeof(struct snmp_answer),0);
	if(num==-1) {
		DEBUG(0,("Error '%s' while sending SNMP Answer\n",strerror(errno)));
		return SNMPCheckError(errno);
	}
}

